/*
	
   
    csci4211 Spring 2006
    Programming Project 1: Messenger with P2P File Sharing System
    Maker Name: Nagapramod Mandagere
    Student ID:
    email address: npramod@cs.umn.edu
*/

/*********************************************
 *
 * File name	    : sockcomm.c 
 * Description      : socket communication program
 *
 **********************************************/

#include "sockcomm.h"


int ConnectToServer(char* hostname, int port) 
{
  int sd; //socket descriptor
  struct sockaddr_in addr; //peer's address
  int ret_val; //return-value
  struct hostent* host_entry; //host-entry
  char** ptr; //pointer to address structure
  /*
    Step 1: Get the complete host address info by making use of hostname argument and appropriate socket function
    Step 2: Create the socket and setup appropriate parameters
    Step 3: Connect to the peer and return this new socket descriptor if successful
   */	

  host_entry = gethostbyname(hostname);
  if(host_entry == NULL) 
    {
      printf("Sorry: Host u r trying to connect to was not found\n");
      exit(1);
    }
  sd = socket(AF_INET, SOCK_STREAM, 0);  
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  addr.sin_addr.s_addr = inet_addr(inet_ntoa(*(struct in_addr *)*host_entry->h_addr_list));
  
  
  
  if(sd < 0)
    {
      perror("ConnectToServer(): socket() error!\n");
      exit(1);
    }

  printf("Peer's hostname is %s, and it's IP address is %s\n", hostname, inet_ntoa(addr.sin_addr));
  ret_val = connect(sd, (struct sockaddr*)&addr, sizeof(addr)); 
  if(ret_val == -1)
    {
      printf("As the peer program is not running on the host \"%s\", the peer on this host cannot connect to the peer on \"%s\".\n", hostname, hostname);
      exit(1);
    }
  
  return(sd);
}



int SocketInit(int port) 
{
  int sd; //socket descriptor
  struct sockaddr_in addr; //socket address structure for accepting join request
  int ret_val; //return-value
  enum boolean flag; //boolean variable
  char hostname[100];
  struct hostent* host_entry;

  /* Step 4: Create a socket
     Step 5: Get own host info by using appropriate socket call
     Step 6: Initialize other socket/address parameters
     Step 7: Bind the socket
     Step 8: Put the socket in listen mode
     Step 9: Return socket descriptor if successful

  */
 
  sd = socket(AF_INET, SOCK_STREAM, 0);  
  if(sd < 0)
  {
     perror("SocketInit(): socket() error!\n");
     exit(1);
  }

  /* avoid bind() error */
  flag = TRUE;
  ret_val = setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, (char*) &flag, sizeof(flag));
  if(ret_val == -1)
  {
     perror("SocketInit(): setsockopt(SO_REUSEADDR) error!\n");
     exit(1);
  }
  
  
  gethostname(hostname,100);
  host_entry = gethostbyname(hostname);
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  addr.sin_addr.s_addr = inet_addr(inet_ntoa(*(struct in_addr *)*host_entry->h_addr_list));
  
  ret_val = bind(sd, (struct sockaddr*) &addr, sizeof(addr));
  



  if(ret_val == -1)
  {
    perror("SocketInit(): bind() error!\n");
    printf("For port:%d\n",port);
    exit(1);
  }
  
  

  
 /* set up the # of backlog */
  ret_val = listen(sd, 5);
  if(ret_val == -1)
  {
     perror("SocketInit(): listen() error!\n");
     exit(1);
  }


  return(sd);
  
}



int AcceptConnection(int sockfd) 
{
  int newsock;
  struct sockaddr_in new_recv_addr; //address of peer node
  int addr_len; //length of sockaddr_in
  u_short new_recv_port; //port# of the new recv socket

  /*
    
    Step 10: Accept connection on 'sockfd'
    Step 11: Return the new socket
  */
  addr_len = sizeof(new_recv_addr); //unless this line, "bad address" error happen!
  newsock = accept(sockfd, 
		   (struct sockaddr *)&new_recv_addr, &addr_len); 
  return newsock;
  
  
}



/* Function: ReadMsg
 * Read a message
 */
int ReadMsg(int fd, char* buff, int size)
{
  int nread;
    
  nread = read(fd, buff, size);
  
  if(nread == -1)
  {
    perror("ReadMsg(): read() error!\n");
    //exit(1);
  }
  
  buff[nread] = '\0';
  return(nread);
}

/* Function: SendMsg
 * receive a message
 */
int SendMsg(int fd, char* buff, int size)
{
  int nwrite;
  
  nwrite = write(fd, buff, size);

  if(nwrite == -1)
  {
    perror("SendMsg(): write() error!\n");
    //exit(1);
  }

  return(nwrite);
}
